/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2008 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.connectors.ws;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.bf.AbstractBindingFactoryPlugin;
import org.objectweb.fractal.bf.BindHints;
import org.objectweb.fractal.bf.BindingFactoryException;
import org.objectweb.fractal.bf.ExportHints;
import org.objectweb.fractal.bf.SkeletonGenerationException;
import org.objectweb.fractal.bf.proxies.StubComponentFactory;
import org.objectweb.fractal.util.Fractal;

/**
 * A WS-Connector can export a Fractal interface to make it available via SOAP.
 * 
 * @author Valerio Schiavoni <valerio.schiavoni@gmail.com>
 * 
 */
public class WsConnector extends
		AbstractBindingFactoryPlugin<WsExportHints, WsBindHints> {

	Logger log = Logger.getLogger(WsConnector.class.getCanonicalName());

	private Component bootstrap;
	private TypeFactory typeFactory;
	private GenericFactory cf;

	/**
	 * Default 0-args constructor.
	 */
	public WsConnector() {
		this.initialHints = new HashMap<String, String>();
		try {
			bootstrap = Fractal.getBootstrapComponent();
			typeFactory = Fractal.getTypeFactory(bootstrap);
			cf = Fractal.getGenericFactory(bootstrap);
		} catch (Exception e) {
			throw new WsConnectorException(e);
		}
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#finalizeSkeleton(org.objectweb.fractal.api.Component,
	 *      java.util.Map)
	 */
	public void finalizeSkeleton(Component skel,
			Map<String, String> finalizationHints)
			throws BindingFactoryException {

	}

	@Override
	public String toString() {
		return "SOAP/CXF Connector Plugin";
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getStubComponentFactory()
	 */
	public StubComponentFactory getStubComponentFactory() {
		return new WsConnectorStubComponentFactoryImpl();
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#createSkel(java.lang.String,
	 *      java.lang.Object, org.objectweb.fractal.api.Component,
	 *      org.objectweb.fractal.bf.ExportHints)
	 */
	public Component createSkel(String interfaceName, Object serverItf,
			Component owner, WsExportHints wsExportHints)
			throws SkeletonGenerationException {
		log.info("WsConnector - Creating skeleton..");

		String address = wsExportHints.getAddress();

		if (address == null) {
			address = WsConnectorConstants.DEFAULT_ADDRESS;
		}

		Component wsServerBridge = null;
		try {

			ComponentType wsServerBridgeType = typeFactory
					.createFcType(new InterfaceType[] {
							typeFactory
									.createFcItfType(
											"attribute-controller",
											"org.objectweb.fractal.bf.connectors.ws.WsConnectorAttributes",
											false, false, false),
							// typeFactory.createFcItfType("service",
							// "org.objectweb.fractal.api.Interface",
							// true, false, false),
							typeFactory.createFcItfType("delegate",
									"org.objectweb.fractal.api.Component",
									true, true, false) });
			wsServerBridge = cf.newFcInstance(wsServerBridgeType, "primitive",
					"org.objectweb.fractal.bf.connectors.ws.WsServerBridge");

			Fractal.getNameController(wsServerBridge).setFcName(
					"org.objectweb.fractal.bf.connectors.ws.WsServerBridge");

			WsConnectorAttributes wsServiceAttributes = (WsConnectorAttributes) Fractal
					.getAttributeController(wsServerBridge);

			InterfaceType it = (InterfaceType) ((Interface) serverItf)
					.getFcItfType();
			Class<?> serviceClass = Thread.currentThread()
					.getContextClassLoader().loadClass(it.getFcItfSignature());
			log.finest("Service class: " + serviceClass
					+ " correctly recovered from context class loader");

			wsServiceAttributes.setAddress(address);
			wsServiceAttributes.setServiceClass(serviceClass);
			wsServiceAttributes.setInterfaceName(interfaceName);

		} catch (Exception e) {
			throw new SkeletonGenerationException(
					"Some error occurred while exporting using the Ws/CXF plugin ",
					e);
		}

		return wsServerBridge;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getExportHints()
	 */
	public ExportHints getExportHints() {
		WsExportHints wsExportHints = new WsExportHints();
		log.fine("Initial hints:" + this.initialHints);
		wsExportHints.setAddress(this.initialHints
				.get(WsConnectorConstants.ADDRESS));

		return wsExportHints;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getPluginIdentifier()
	 */
	public String getPluginIdentifier() {
		return "ws";
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getBindHints()
	 */
	public BindHints getBindHints() {
		WsBindHints wsBindHints = new WsBindHints();
		wsBindHints.setAddress(this.initialHints
				.get(WsConnectorConstants.ADDRESS));
		log.fine("WsBindHints: " + wsBindHints);
		return wsBindHints;
	}

}
